/*
 * Copyright (C) 2008 Adam Harwell, all rights reserved.
 *
 * Contact: Adam Harwell <adam.harwell@trinity.edu>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtk/gtk.h>
#include <glib-object.h>

#include <libhildondesktop/libhildondesktop.h>
#include <hildon/hildon-controlbar.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "advanced-backlight.h"

/* Utility macro which defines the plugin dynamic type bits */
HD_DEFINE_PLUGIN (AdvancedBacklightPlugin, advanced_backlight_plugin, STATUSBAR_TYPE_ITEM);

static void platform_check();
static void pressed(GtkToggleButton *button);
static void set_level_dsme(GtkRange *range, gpointer user_data);

/////////////////////////////////////////////////////////////////////////////////////////////////////
//// I'm new to GTK so I'm not sure how to pass certain things around... So they're global. Ick. ////
/////////////////////////////////////////////////////////////////////////////////////////////////////
GtkWidget* window;
int lastvalue;
GtkWidget *button;

char oldfile[66]; // To keep the statusbar icon up to date, I keep track of the current icon (oldfile)
                  // so I can update only when it is different (no flickering icon).
int n810 = 0; // Check for n810 hardware to allow only n810 owners to turn off backlight completely.

/////////////////////////////////////////////////////////////////////
//// This is the initialization function. It is called on entry. ////
/////////////////////////////////////////////////////////////////////
static void advanced_backlight_plugin_init(AdvancedBacklightPlugin *statusbar_plugin)
{
	//GtkWidget *button; // The button wasn't always global...

	platform_check(); // See if we are on an n810

	button = gtk_button_new(); // The statusbar icon...

	gtk_button_set_image(GTK_BUTTON(button), gtk_image_new_from_file("/usr/share/icons/hicolor/40x40/hildon/qgn_stat_displaybright4.png"));
	gtk_widget_show_all(button);
	gtk_container_add(GTK_CONTAINER(statusbar_plugin), button); // Shove the button into the statusbar
	g_signal_connect(button, "button-press-event", G_CALLBACK(pressed),NULL); // On click it will call the function "pressed()".

	FILE* myfile = fopen("/sys/devices/platform/omapfb/panel/backlight_level","r");
	fscanf(myfile,"%d",&lastvalue); // A failed attempt to get the current backlight level. Gets only the software level...
	fclose(myfile);
}

//////////////////////////////////////////////////
//// Check to see if we are on an n810 or not ////
//////////////////////////////////////////////////
static void platform_check()
{
	char version;

	FILE * fp = fopen("/proc/component_version","r");
	fseek(fp,sizeof(char)*15,0); // The important character is the 16th in the file.
	fread(&version,sizeof(char),1,fp);
	fclose(fp);

	if (version == '4') // 4 means n810, 3 means n800.
		n810 = 1;
	else // Err on the side of caution.
		n810 = 0;
}

/////////////////////////////////////////////////////////////////////////////////////////////////////
//// Called on statusbar icon click. Pops a whole window at the moment, which is less than ideal ////
/////////////////////////////////////////////////////////////////////////////////////////////////////
static void pressed(GtkToggleButton *btn)
{
	gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_NONE);
	if (window != NULL) {	// If the window is already initialized, we want to kill it.
		gtk_widget_destroy(window);
		gtk_button_set_relief(GTK_BUTTON(btn), GTK_RELIEF_NORMAL);
		window = NULL;
		return; // There is probably a more efficient way to do this, like just hiding the window...
	}

	window = gtk_window_new(GTK_WINDOW_POPUP); // This window is global so that we can check if it is open (above).
	GtkWidget *cbar = hildon_controlbar_new();

	if (n810)	// Only let n810 set brightness 0, since they have a transflexive screen
		hildon_controlbar_set_range(HILDON_CONTROLBAR (cbar), 0, 127);
	else		// An n800 user would lose their backlight entirely and be confused
		hildon_controlbar_set_range(HILDON_CONTROLBAR (cbar), 1, 127);

	hildon_controlbar_set_value(HILDON_CONTROLBAR(cbar), lastvalue); // Return the controlbar to the last specified value.
	gtk_range_set_update_policy(GTK_RANGE(cbar), GTK_UPDATE_DELAYED);

	GtkWidget* hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(hbox), cbar, 0,0,30);
	gtk_widget_show(cbar);
	
	GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, 0,0,30);
	gtk_widget_show(hbox);

	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_widget_show(vbox);

	GtkRequisition req;
	gtk_widget_size_request(GTK_WIDGET(window), &req);
	gint sw = gdk_screen_get_width(gtk_widget_get_screen(window));

	gint x,y;
	gdk_window_get_position(GTK_WIDGET(btn)->window, &x, &y);

	y += GTK_WIDGET(btn)->allocation.y + GTK_WIDGET(btn)->allocation.height + 12;
	x += GTK_WIDGET(btn)->allocation.x;

	if (x + req.width > sw) {
		x -= req.width - GTK_WIDGET(btn)->allocation.width;
	}

	gtk_window_move(GTK_WINDOW(window), x, y);
	gtk_window_set_has_frame(GTK_WINDOW(window),TRUE);

	gtk_widget_show(window);
	
	g_signal_connect(cbar, "value-changed", G_CALLBACK(set_level_dsme), NULL); // When the slider moves, call the function "set_level_dsme()"
}

/////////////////////////////////////////////////////////
//// Does the actual work for setting the backlight. ////
/////////////////////////////////////////////////////////
static void set_level_dsme(GtkRange *range, gpointer user_data)
{
	gdouble myval = gtk_range_get_value(range); // Get the value out of the GtkRange.
	int myint = floor(myval); // We need an integer, not a double.
	lastvalue=myint; // Keep track of the last value set.

	myint = myint * 2 + 1;

	char command[80] = "sudo /usr/sbin/chroot /mnt/initfs/ dsmetest -l ";

	char brightness[4];
	snprintf(brightness, 4, "%d", myint);
	strncat(command, brightness, 4);
	//strncat(command, " 2>> /home/user/dsmelog", 23);
	g_spawn_command_line_async(command,NULL);

	/*
	GtkWidget * dialog = gtk_message_dialog_new (NULL,
			GTK_DIALOG_DESTROY_WITH_PARENT,
			GTK_MESSAGE_ERROR,
			GTK_BUTTONS_CLOSE, state);
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
	*/

	// Propose a statusbar icon change based on the new brightness level. Available images are 1-4, empty-full.
	char newfile[66];
	sprintf(newfile,"/usr/share/icons/hicolor/40x40/hildon/qgn_stat_displaybright%d.png", (int)(ceil((double)lastvalue / 32.0) + 0.2));

	if (strcmp(oldfile,newfile) != 0) { // If proposed icon is different than the current one, go ahead and change it.
		gtk_button_set_image(GTK_BUTTON (button), gtk_image_new_from_file(newfile));
		strncpy(oldfile,newfile,66); // Keep track of the current icon after the change.
	}

}

////////////////////////////////////////////////////////////////////////////
//// This is necessary, according to the statusbar template I followed. ////
////////////////////////////////////////////////////////////////////////////
static void advanced_backlight_plugin_class_init(AdvancedBacklightPluginClass *class)
{
}
